﻿XIncludeFile "include\ConvenientMathMacros.pbi"
XIncludeFile "include\Multiords.pbi"
XIncludeFile "include\ArrayCollision.pbi"


Structure Phys2D
  Array datCollision.a(0, 0)
  
  posPosition.CoordF
  posVelocity.CoordF
  
  parElasticity.CoordF
  parFrictionGround.CoordF
  parFrictionAir.CoordF
  parWeight.CoordF
  
  retCollision.CoordL
  
EndStructure

Structure Entity2D
  Phys.Phys2D
  GraphicsSprite.i
EndStructure


Procedure MovePhys2D(*Active.Phys2D, *Passive.Phys2D, ApplyVelocity.l = 0)
  ;This procedure allows players to walk up stairs.
  ;*Active.PlatformObject - This object will move, and apply velocity to *Passive if applicable.
  ;*Passive.PlatformObject - This object acts as the stator, or "world", if you will.
  
  Protected Accuracy.i ;This will be used to determine the amount of steps required to compute the position of *Active. 
  
  Protected CurrentPosition.CoordF
  Protected CurrentVelocity.CoordF
  Protected CurrentCollision.Coord
  
  Protected CurrentStep.CoordF
  
  ;Determine, whether X or Y has the bigger velocity, then set Accuracy to the rounded-up value of the higher velocity.
  Accuracy = Round(Max2F(Abs(*Active\posVelocity\X), Abs(*Active\posVelocity\Y)), #PB_Round_Up)
  
  If Accuracy = 0
    ProcedureReturn #False
  EndIf
  
  ;The Ratio of either field will never be greater than 1.
  CurrentVelocity\X = *Active\posVelocity\X/Accuracy
  CurrentVelocity\Y = *Active\posVelocity\Y/Accuracy
  
  *Active\posPosition\X-CurrentVelocity\X
  *Active\posPosition\Y-CurrentVelocity\Y
  
  
  For CollisionStep = 0 To Accuracy
    *Active\posPosition\X+CurrentVelocity\X
    *Active\posPosition\Y+CurrentVelocity\Y
    
    If ArrayCollision2D(*Active\datCollision(), *Active\posPosition\X+Sign(CurrentVelocity\X), *Active\posPosition\Y, *Passive\datCollision(), *Passive\posPosition\X, *Passive\posPosition\Y)
      If Not ArrayCollision2D(*Active\datCollision(), *Active\posPosition\X+Sign(CurrentVelocity\X), *Active\posPosition\Y-1, *Passive\datCollision(), *Passive\posPosition\X, *Passive\posPosition\Y); And *Active\posVelocity\Y >= 0
        *Active\posPosition\Y-1
      Else
        *Active\posPosition\X-CurrentVelocity\X
        CurrentCollision\X = 1
      EndIf
    EndIf
    
    If ArrayCollision2D(*Active\datCollision(), *Active\posPosition\X, *Active\posPosition\Y+Sign(CurrentVelocity\Y), *Passive\datCollision(), *Passive\posPosition\X, *Passive\posPosition\Y)
      *Active\posPosition\Y-CurrentVelocity\Y
      CurrentCollision\Y = 1
    EndIf
  Next
  
  *Active\posPosition\X+CurrentVelocity\X
  *Active\posPosition\Y+CurrentVelocity\Y
  
  If CurrentCollision\X
    If ApplyVelocity = 1
      *Passive\posVelocity\X+*Active\posVelocity\X**Active\parWeight\X
    EndIf
    *Active\retCollision\X = Sign(CurrentVelocity\X)
    *Active\posVelocity\X = -*Active\posVelocity\X**Active\parElasticity\X
    
  Else
    *Active\retCollision\X = 0
  EndIf
  
  If CurrentCollision\Y
    If ApplyVelocity = 1
      *Passive\posVelocity\Y+*Active\posVelocity\Y**Active\parWeight\Y
    EndIf
    *Active\retCollision\Y = Sign(CurrentVelocity\Y)
    *Active\posVelocity\Y = -*Active\posVelocity\Y**Active\parElasticity\Y
  Else
    *Active\retCollision\Y = 0
  EndIf
  ProcedureReturn CurrentCollision\X|CurrentCollision\Y
  
EndProcedure

Procedure FastMovePhys2D(*Active.Phys2D, *Passive.Phys2D)
  ;This procedure does not have stair-recognizing code.
  ;*Active.PlatformObject - This object will move, and apply velocity to *Passive if applicable.
  ;*Passive.PlatformObject - This object acts as the stator, or "world", if you will.
  
  Protected Accuracy.i ;This will be used to determine the amount of steps required to compute the position of *Active. 
  
  Protected CurrentPosition.CoordF
  Protected CurrentVelocity.CoordF
  Protected CurrentCollision.Coord
  
  Protected CurrentStep.CoordF
  
  ;Determine, whether X or Y has the bigger velocity, then set Accuracy to the rounded-up value of the higher velocity.
  Accuracy = Round(Max2F(Abs(*Active\posVelocity\X), Abs(*Active\posVelocity\Y)), #PB_Round_Up)
  
  If Accuracy = 0
    ProcedureReturn #False
  EndIf
  
  ;The Ratio of either field will never be greater than 1.
  CurrentVelocity\X = *Active\posVelocity\X/Accuracy
  CurrentVelocity\Y = *Active\posVelocity\Y/Accuracy
  
  *Active\posPosition\X-CurrentVelocity\X
  *Active\posPosition\Y-CurrentVelocity\Y
  
  
  For CollisionStep = 0 To Accuracy
    *Active\posPosition\X+CurrentVelocity\X
    *Active\posPosition\Y+CurrentVelocity\Y
    
    If ArrayCollision2D(*Active\datCollision(), *Active\posPosition\X+Sign(CurrentVelocity\X), *Active\posPosition\Y, *Passive\datCollision(), *Passive\posPosition\X, *Passive\posPosition\Y)
      *Active\posPosition\X-CurrentVelocity\X
      CurrentCollision\X = 1
    EndIf
    
    If ArrayCollision2D(*Active\datCollision(), *Active\posPosition\X, *Active\posPosition\Y+Sign(CurrentVelocity\Y), *Passive\datCollision(), *Passive\posPosition\X, *Passive\posPosition\Y)
      *Active\posPosition\Y-CurrentVelocity\Y
      CurrentCollision\Y = 1
    EndIf
  Next
  
  *Active\posPosition\X+CurrentVelocity\X
  *Active\posPosition\Y+CurrentVelocity\Y
  
  If CurrentCollision\X
    *Active\retCollision\X = Sign(CurrentVelocity\X)
    *Active\posVelocity\X = -*Active\posVelocity\X**Active\parElasticity\X
  Else
    *Active\retCollision\X = 0
  EndIf
  
  If CurrentCollision\Y
    *Active\retCollision\Y = Sign(CurrentVelocity\Y)
    *Active\posVelocity\Y = -*Active\posVelocity\Y**Active\parElasticity\Y
  Else
    *Active\retCollision\Y = 0
  EndIf
  ProcedureReturn CurrentCollision\X|CurrentCollision\Y
  
EndProcedure


; IDE Options = PureBasic 5.24 LTS (Windows - x86)
; CursorPosition = 7
; Folding = -
; Executable = ..\main.exe
; CPU = 5
; CurrentDirectory = ..\
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant