﻿Structure FileEntry
  PackID.i
  InternalFile$
  Size.i
EndStructure

UseZipPacker()


Procedure CreateIndex(Map FileIndex.FileEntry(), Directory$, SearchZipDepth.l = 1)
  ;.zip are loaded to the base directory.
  ;example.zip in subdirectory "base\" with contents "info.png" and "dir\info2.png
  ;will be loaded as "base\info.png" and "base\dir\info2.png".
  ;All file names will be converted to lowercase.
  ;Nested .zip are not supported.
  
  Protected DirID.i
  Protected PackID.i
  Protected PackEntry$
  
  ;Dynamic for recursion.
  DirID = ExamineDirectory(#PB_Any, Directory$, "*")
  Debug "dir:"+Directory$
  While NextDirectoryEntry(DirID)
    
    ;Filtering out ThisDir (= ".") and AboveDir (= "..")
    If DirectoryEntryName(DirID) = "." Or DirectoryEntryName(DirID) = ".."
      Continue
    EndIf
    
    Select DirectoryEntryType(DirID)
      Case #PB_DirectoryEntry_Directory
        ;Call self to index every directory.
        CreateIndex(FileIndex(), Directory$+DirectoryEntryName(DirID)+"\", SearchZipDepth-1)
      Case #PB_DirectoryEntry_File
        If LCase(GetExtensionPart(DirectoryEntryName(DirID))) = "zip"
          If SearchZipDepth.l <> 0
            PackID = OpenPack(#PB_Any, Directory$+DirectoryEntryName(DirID))
            If PackID
              Debug "zip:"+Directory$+DirectoryEntryName(DirID)
              ExaminePack(PackID)
              While NextPackEntry(PackID)
                Select PackEntryType(PackID)
                  Case #PB_Packer_File
                    PackEntry$ = LCase(Directory$+ReplaceString(PackEntryName(PackID), "/", "\"))
                    If FindMapElement(FileIndex(), LCase(PackEntry$))
                      If FileIndex(PackEntry$)\PackID = 0
                        Debug "exi:"+PackEntry$
                        Continue
                      EndIf
                    EndIf
                    Debug "pak:"+PackEntry$
                    FileIndex(PackEntry$)\PackID = PackID
                    FileIndex(PackEntry$)\InternalFile$ = PackEntryName(PackID)
                    FileIndex(PackEntry$)\Size = PackEntrySize(PackID, #PB_Packer_UncompressedSize)
                EndSelect
              Wend
            EndIf
          EndIf
        EndIf 
        Debug "fil:"+LCase(Directory$+DirectoryEntryName(DirID))
        ;Debug "rlp:"+GetCurrentDirectory()+Directory$+DirectoryEntryName(DirID)
        FileIndex(LCase(Directory$+DirectoryEntryName(DirID)))\PackID = 0
        FileIndex(LCase(Directory$+DirectoryEntryName(DirID)))\InternalFile$ = GetCurrentDirectory()+Directory$+DirectoryEntryName(DirID)
        FileIndex(LCase(Directory$+DirectoryEntryName(DirID)))\Size = DirectoryEntrySize(DirID)
        
        
    EndSelect
  Wend
  FinishDirectory(DirID)
EndProcedure

Procedure LoadIndexedSprite(Map FileIndex.FileEntry(), Sprite.i, File$, Flags.i = 0)
  Protected *ReturnValue
  Protected *Buffer
  File$ = LCase(File$)
  If FindMapElement(FileIndex(), File$)
    Select FileIndex(File$)\PackID
      Case 0
        *ReturnValue = LoadSprite(Sprite, FileIndex(File$)\InternalFile$, Flags)
      Default
        *Buffer = AllocateMemory(FileIndex(File$)\Size)
        UncompressPackMemory(FileIndex(File$)\PackID, *Buffer, FileIndex(File$)\Size, FileIndex(File$)\InternalFile$)
        *ReturnValue = CatchSprite(Sprite, *Buffer, Flags)
        FreeMemory(*Buffer)
    EndSelect
  EndIf
  ProcedureReturn *ReturnValue
EndProcedure
Procedure LoadIndexedSound(Map FileIndex.FileEntry(), Sound.i, File$, Flags.i = 0)
  Protected *ReturnValue
  Protected *Buffer
  File$ = LCase(File$)
  If FindMapElement(FileIndex(), File$)
    Select FileIndex(File$)\PackID
      Case 0
        *ReturnValue = LoadSound(Sound, FileIndex(File$)\InternalFile$, Flags)
      Default
        *Buffer = AllocateMemory(FileIndex(File$)\Size)
        UncompressPackMemory(FileIndex(File$)\PackID, *Buffer, FileIndex(File$)\Size, FileIndex(File$)\InternalFile$)
        *ReturnValue = CatchSound(Sound, *Buffer, FileIndex(File$)\Size, Flags)
        FreeMemory(*Buffer)
    EndSelect
  EndIf
  ProcedureReturn *ReturnValue
EndProcedure
Procedure LoadIndexedImage(Map FileIndex.FileEntry(), Image.i, File$)
  Protected *ReturnValue
  Protected *Buffer
  File$ = LCase(File$)
  If FindMapElement(FileIndex(), File$)
    Select FileIndex(File$)\PackID
      Case 0
        *ReturnValue = LoadImage(Image, FileIndex(File$)\InternalFile$)
      Default
        *Buffer = AllocateMemory(FileIndex(File$)\Size)
        UncompressPackMemory(FileIndex(File$)\PackID, *Buffer, FileIndex(File$)\Size, FileIndex(File$)\InternalFile$)
        *ReturnValue = CatchImage(Image, *Buffer, FileIndex(File$)\Size)
        FreeMemory(*Buffer)
    EndSelect
  EndIf
  ProcedureReturn *ReturnValue
EndProcedure
Procedure LoadIndexedBuffer(Map FileIndex.FileEntry(), File$)
  Protected *ReturnValue
  Protected FileNo.i
  File$ = LCase(File$)
  If FindMapElement(FileIndex(), File$)
    Select FileIndex(File$)\PackID
      Case 0
        *ReturnValue = AllocateMemory(FileSize(File$))
        FileNo = ReadFile(#PB_Any, FileIndex(File$)\InternalFile$)
        ReadData(FileNo, *ReturnValue, FileSize(File$))
        CloseFile(FileNo)
      Default
        *ReturnValue = AllocateMemory(FileIndex(File$)\Size)
        UncompressPackMemory(FileIndex(File$)\PackID, *ReturnValue, FileIndex(File$)\Size, FileIndex(File$)\InternalFile$)
    EndSelect
  EndIf
  ProcedureReturn *ReturnValue
EndProcedure
; IDE Options = PureBasic 5.24 LTS (Windows - x86)
; CursorPosition = 60
; FirstLine = 57
; Folding = -
; EnableCompileCount = 0
; EnableBuildCount = 0
; EnableExeConstant